﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Threading.Tasks;
using System.Net;
using System.Web;
using System.Web.Mvc;
using GamesStore.Entities;
using GamesStore.Web.DataProviders;
using GamesStore.Web.Models.Games;

using AutoMapper;
using System.Threading;

namespace GamesStore.Web.Controllers
{
    public class GamesController : BaseGamesController
    {
        // GET: Games
        public async Task<ActionResult> Index(string filter)
        {
            IEnumerable<Game> games;
            if (filter != null)
            {
                var genres = filter.Split(';')
                    .Select(s => Enum.Parse(typeof(GameGenreEnum), s))
                    .Cast<GameGenreEnum>();

                games = await GamesDataProvider.Games
                    .Where(g => genres.Contains(g.Genre))
                    .ToListAsync();
            }
            else
            {
                games = await GamesDataProvider.Games.ToListAsync();
            }

            return View(Mapper.Map<IEnumerable<GameViewModel>>(games));
        }

        // GET: Games/Details/5
        public async Task<ActionResult> Details(Guid? id)
        {
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }
            Game game = await GamesDataProvider.Games.FindAsync(id);
            if (game == null)
            {
                return HttpNotFound();
            }

            return View(Mapper.Map<GameViewModel>(game));
        }

        [Authorize]
        public ActionResult Create()
        {
            return View();
        }


        // POST: Games/Create
        // To protect from overposting attacks, please enable the specific properties you want to bind to, for 
        // more details see http://go.microsoft.com/fwlink/?LinkId=317598.
        [HttpPost]
        [ValidateAntiForgeryToken]
        [Authorize]
        public async Task<ActionResult> Create(GameWithoutIdViewModel newGame)
        {
            if (ModelState.IsValid)
            {
                var dbGame = Mapper.Map<Game>(newGame);
                dbGame.Id = Guid.NewGuid();
                GamesDataProvider.Games.Add(dbGame);
                await GamesDataProvider.SaveChangesAsync();
                return RedirectToAction("Index");
            }

            return View(newGame);
        }

        // GET: Games/Edit/5
        [Authorize]
        public async Task<ActionResult> Edit(Guid? id)
        {
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }

            var game = await GamesDataProvider.Games.FindAsync(id);
            if (game == null)
            {
                return HttpNotFound();
            }

            return View(Mapper.Map<GameViewModel>(game));
        }

        // POST: Games/Edit/5
        // To protect from overposting attacks, please enable the specific properties you want to bind to, for 
        // more details see http://go.microsoft.com/fwlink/?LinkId=317598.
        [HttpPost]
        [ValidateAntiForgeryToken]
        [Authorize]
        public async Task<ActionResult> Edit(GameViewModel game)
        {
            if (ModelState.IsValid)
            {
                GamesDataProvider.Entry(Mapper.Map<Game>(game)).State = EntityState.Modified;
                await GamesDataProvider.SaveChangesAsync();
                return RedirectToAction("Index");
            }

            return View(game);
        }

        [Authorize]
        public async Task<ActionResult> Delete(Guid? id)
        {
            if (id == null)
            {
                return new HttpStatusCodeResult(HttpStatusCode.BadRequest);
            }
            Game game = await GamesDataProvider.Games.FindAsync(id);
            if (game == null)
            {
                return HttpNotFound();
            }

            return View(Mapper.Map<GameViewModel>(game));
        }

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        [Authorize]
        public async Task<ActionResult> DeleteConfirmed(Guid id)
        {
            Game game = await GamesDataProvider.Games.FindAsync(id);
            GamesDataProvider.Games.Remove(game);
            await GamesDataProvider.SaveChangesAsync();
            return RedirectToAction("Index");
        }


        [HttpGet]
        public ActionResult FilterArray()
        {
            var values = (GameGenreEnum[])Enum.GetValues(typeof(GameGenreEnum));
            return View(values);
        }


        [HttpPost]
        public ActionResult FilterArray(bool[] genresOptions)
        {
            if (genresOptions == null)
            {
                return RedirectToAction("index");
            }

            var genres = ((GameGenreEnum[])Enum.GetValues(typeof(GameGenreEnum)));
            List<GameGenreEnum> selectedGenres = new List<GameGenreEnum>();
            for (int i = 0; i < genresOptions.Length; i++)
            {
                if (genresOptions[i])
                {
                    selectedGenres.Add(genres[i]);
                }
            }

            return RedirectToAction("Index", new { Filter = String.Join(";", selectedGenres) });
        }


        [HttpGet]
        public ActionResult FilterIndexedArray()
        {
            var values = new GenreSelectionArrayViewModel
            {
                Genres = ((GameGenreEnum[])Enum.GetValues(typeof(GameGenreEnum)))
                .Select(g => new SelectedGenreViewModel
                {
                    Genre = g,
                    IsChecked = true,
                })
                .ToArray()
            };

            return View(values);
        }


        [HttpGet]
        public ActionResult FilterIndexedArrayWithTemplate()
        {
            var values = new GenreSelectionArrayViewModel
            {
                Genres = ((GameGenreEnum[])Enum.GetValues(typeof(GameGenreEnum)))
                .Select(g => new SelectedGenreViewModel
                {
                    Genre = g,
                    IsChecked = true,
                })
                .ToArray()
            };

            return View(values);
        }


        [HttpPost]
        public ActionResult FilterIndexedArray(GenreSelectionArrayViewModel genresSelection)
        {
            return FilterIndexedArrayInternal(genresSelection);
        }


        [HttpPost]
        public ActionResult FilterIndexedArrayWithTemplate(GenreSelectionArrayViewModel genresSelection)
        {
            return FilterIndexedArrayInternal(genresSelection);
        }


        private ActionResult FilterIndexedArrayInternal(GenreSelectionArrayViewModel genresSelection)
        {
            if (genresSelection == null)
            {
                return RedirectToAction("index");
            }

            var selectedGenres = genresSelection.Genres.Where(g => g.IsChecked).Select(g => g.Genre).ToList();

            return RedirectToAction("Index", new { Filter = String.Join(";", selectedGenres) });
        }

        [HttpGet]
        public ActionResult FilterDictionary()
        {
            var values = new GenreSelectionDictionaryViewModel
            {
                Genres = ((GameGenreEnum[])Enum.GetValues(typeof(GameGenreEnum)))
                    .ToDictionary(g => g, g => true),
            };

            return View(values);
        }


        [HttpPost]
        public ActionResult FilterDictionary(GenreSelectionDictionaryViewModel genresSelection)
        {
            if (genresSelection == null)
            {
                return RedirectToAction("index");
            }

            var selectedGenres = genresSelection.Genres.Where(pair => pair.Value).Select(pair => pair.Key).ToList();

            return RedirectToAction("Index", new { Filter = String.Join(";", selectedGenres) });
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                GamesDataProvider.Dispose();
            }
            base.Dispose(disposing);
        }
    }
}
